(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     11699,        350]
NotebookOptionsPosition[     10549,        308]
NotebookOutlinePosition[     10935,        325]
CellTagsIndexPosition[     10892,        322]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["FixPolygons example", "Title",
 CellChangeTimes->{{3.4014173040803585`*^9, 3.40141730947812*^9}}],

Cell[BoxData[
 RowBox[{"SetDirectory", "@", 
  RowBox[{"NotebookDirectory", "[", "]"}]}]], "Input",
 CellChangeTimes->{{3.404537419044571*^9, 3.404537426104723*^9}}],

Cell[BoxData[
 RowBox[{"<<", "FixPolygons`"}]], "Input"],

Cell[CellGroupData[{

Cell["The problem", "Section",
 CellChangeTimes->{{3.4009947145183983`*^9, 3.400994716401106*^9}, {
  3.4011612797945104`*^9, 3.401161281366771*^9}, {3.4014173405027313`*^9, 
  3.4014173432667055`*^9}, {3.4014251192680464`*^9, 3.4014251225527697`*^9}, {
  3.401425223958584*^9, 3.4014252241088*^9}}],

Cell["\<\
ContourPlot and RegionPlot create 2D plots with as many polygons as elements \
in the data mesh. This is inefficent when most of the polygons will be \
adjacent and the same colour. Furthermore, exported images are ugly and slow \
to render in EPS and PDF (at least).\
\>", "Text"],

Cell["\<\
Rather than saving the figure and importing it, the same effect can be seen \
by highlighting the mesh:\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"g", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"a", "^", "2"}], "+", 
      RowBox[{"b", "^", "2"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", 
       RowBox[{"-", "10"}], ",", "10"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"b", ",", 
       RowBox[{"-", "5"}], ",", "5"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"g", "/.", 
  RowBox[{
   RowBox[{"EdgeForm", "[", "]"}], "\[Rule]", 
   RowBox[{"EdgeForm", "[", 
    RowBox[{"{", "Black", "}"}], "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.400994721838925*^9, 3.4009947856106243`*^9}, {
  3.417779445550868*^9, 3.417779491292917*^9}, {3.417779576417308*^9, 
  3.417779668059332*^9}, {3.417779720904002*^9, 3.417779744778289*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The solution", "Section",
 CellChangeTimes->{{3.4011562692497005`*^9, 3.4011562697704496`*^9}, {
  3.401158797384981*^9, 3.4011588000187683`*^9}, {3.4011612403277597`*^9, 
  3.4011612423106112`*^9}, {3.4011612841207314`*^9, 3.401161286764533*^9}, {
  3.40141125267887*^9, 3.401411283433093*^9}, {3.4014175490325823`*^9, 
  3.4014175515161533`*^9}, {3.4014251804159727`*^9, 3.4014251828594866`*^9}}],

Cell[TextData[{
 "The seemingly simple solution is to \"just\" merge adjacent polygons of the \
same colour. \nThe ",
 StyleBox["FixPolygon", "Input"],
 " function does just that."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"g2", "=", 
   RowBox[{
    RowBox[{"ContourPlot", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"x", "^", "2"}], "+", 
       RowBox[{"y", "^", "2"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", 
        RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"y", ",", 
        RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}], "//", 
    "FixPolygons"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"g2", "/.", 
  RowBox[{
   RowBox[{"EdgeForm", "[", "]"}], "\[Rule]", 
   RowBox[{"EdgeForm", "[", 
    RowBox[{"{", "Black", "}"}], "]"}]}]}]}], "Input"],

Cell[CellGroupData[{

Cell["The EarlyCleanupThreshold option", "Subsection",
 CellChangeTimes->{{3.417783012225625*^9, 3.417783017263445*^9}}],

Cell[TextData[{
 "A tradeoff between speed of processing and quality/size of the final plot \
can be tuned by the ",
 StyleBox["EarlyCleanupThreshold", "Input"],
 " option. At each stage of the recursive cleanup, vertices shared by less \
than ",
 StyleBox["EarlyCleanupThreshold", "Input"],
 " polygons are deleted from the refinement list. Higher values means that \
less pattern matching must be performed, but more mergeable polygons might be \
left behind."
}], "Text",
 CellChangeTimes->{{3.417783025831254*^9, 3.417783069980251*^9}, {
  3.417783101359017*^9, 3.417783181491633*^9}, {3.417783220699436*^9, 
  3.417783281913442*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"npoly", "[", "graph_", "]"}], ":=", 
  RowBox[{"Length", "[", 
   RowBox[{
    RowBox[{"Cases", "[", 
     RowBox[{
      RowBox[{"graph", "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "2"}], "]"}], "]"}], ",", "x_Polygon", ",", 
      "Infinity"}], "]"}], "/.", 
    RowBox[{
     RowBox[{"Polygon", "[", 
      RowBox[{
       RowBox[{"{", "a__", "}"}], ",", "b___"}], "]"}], ":>", "a"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.417782765249264*^9, 3.417782863402372*^9}, {
  3.41778289496252*^9, 3.417782958883612*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"g", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"a", "^", "2"}], "+", 
      RowBox[{"b", "^", "2"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", 
       RowBox[{"-", "10"}], ",", "10"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"b", ",", 
       RowBox[{"-", "5"}], ",", "5"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g", "/.", 
   RowBox[{
    RowBox[{"EdgeForm", "[", "]"}], "\[Rule]", 
    RowBox[{"EdgeForm", "[", 
     RowBox[{"{", "Black", "}"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"npoly", "[", "g", "]"}]}], "Input",
 CellChangeTimes->{{3.400994721838925*^9, 3.4009947856106243`*^9}, {
  3.417779445550868*^9, 3.417779491292917*^9}, {3.417779576417308*^9, 
  3.417779668059332*^9}, {3.417779720904002*^9, 3.417779744778289*^9}, {
  3.417783311049794*^9, 3.417783316520568*^9}}],

Cell[BoxData[{
 RowBox[{"Timing", "[", 
  RowBox[{
   RowBox[{"g2", "=", 
    RowBox[{"FixPolygons", "[", 
     RowBox[{"g", ",", 
      RowBox[{"EarlyCleanupThreshold", "\[Rule]", "1"}]}], "]"}]}], ";"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"g2", "/.", 
  RowBox[{
   RowBox[{"EdgeForm", "[", "]"}], "\[Rule]", 
   RowBox[{"EdgeForm", "[", 
    RowBox[{"{", "Black", "}"}], "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"npoly", "[", "g2", "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"Timing", "[", 
  RowBox[{
   RowBox[{"g2", "=", 
    RowBox[{"FixPolygons", "[", 
     RowBox[{"g", ",", 
      RowBox[{"EarlyCleanupThreshold", "\[Rule]", "3"}]}], "]"}]}], ";"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"g2", "/.", 
  RowBox[{
   RowBox[{"EdgeForm", "[", "]"}], "\[Rule]", 
   RowBox[{"EdgeForm", "[", 
    RowBox[{"{", "Black", "}"}], "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"npoly", "[", "g2", "]"}]}], "Input"],

Cell["\<\
Sometimes performing sequentially several FixPolygons calls at better \
thresholds leads to slight speed improvement (it might be interesting to \
explore this direction to gain further speedup at high quality)\
\>", "Text",
 CellChangeTimes->{{3.417783546749363*^9, 3.417783616132163*^9}, {
  3.417785294920905*^9, 3.417785311926357*^9}}],

Cell[BoxData[{
 RowBox[{"Timing", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"g2", "=", 
    RowBox[{"FixPolygons", "[", 
     RowBox[{"g", ",", 
      RowBox[{"EarlyCleanupThreshold", "\[Rule]", "5"}]}], "]"}]}], ";", 
   "\[IndentingNewLine]", 
   RowBox[{"g2", "=", 
    RowBox[{"FixPolygons", "[", 
     RowBox[{"g2", ",", 
      RowBox[{"EarlyCleanupThreshold", "\[Rule]", "3"}]}], "]"}]}], ";", 
   "\[IndentingNewLine]", 
   RowBox[{"g2", "=", 
    RowBox[{"FixPolygons", "[", 
     RowBox[{"g2", ",", 
      RowBox[{"EarlyCleanupThreshold", "\[Rule]", "1"}]}], "]"}]}], ";"}], 
  "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", 
 RowBox[{"g2", "/.", 
  RowBox[{
   RowBox[{"EdgeForm", "[", "]"}], "\[Rule]", 
   RowBox[{"EdgeForm", "[", 
    RowBox[{"{", "Black", "}"}], "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"npoly", "[", "g2", "]"}]}], "Input",
 CellChangeTimes->{{3.41777978844889*^9, 3.417779790824435*^9}, {
   3.417781799856523*^9, 3.417781818186475*^9}, {3.417781870241157*^9, 
   3.417782005819804*^9}, {3.417782088138774*^9, 3.417782105803808*^9}, {
   3.417782307565774*^9, 3.417782366481011*^9}, {3.417782519094878*^9, 
   3.417782534878772*^9}, {3.417782738443322*^9, 3.417782748502438*^9}, {
   3.41778332986626*^9, 3.417783342240204*^9}, {3.41778339076214*^9, 
   3.417783412553007*^9}, {3.417783454589423*^9, 3.417783496427962*^9}, 
   3.417783540620137*^9, {3.41778362880112*^9, 3.417783774980615*^9}, 
   3.417783909216893*^9, {3.417783948540423*^9, 3.417783976783626*^9}}],

Cell["\<\
this example shows clearly why higher values of EarlyCleanupThreshold leave \
some polygons behind\
\>", "Text",
 CellChangeTimes->{{3.417783853768509*^9, 3.417783902180658*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"test", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"EdgeForm", "[", 
       RowBox[{"{", 
        RowBox[{"Black", ",", "Thick"}], "}"}], "]"}], ",", 
      RowBox[{"FaceForm", "[", "LightBlue", "]"}], ",", 
      RowBox[{"GraphicsComplex", "[", 
       RowBox[{
        RowBox[{"Flatten", "[", 
         RowBox[{
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"i", ",", "j"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "0", ",", "3"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"j", ",", "0", ",", "2"}], "}"}]}], "]"}], ",", "1"}], 
         "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"Polygon", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"1", ",", "3", ",", "5", ",", "4"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"5", ",", "3", ",", "9", ",", "8"}], "}"}]}], "}"}], 
          "]"}], "}"}]}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Timing", "@", 
  RowBox[{"FixPolygons", "[", 
   RowBox[{"test", ",", 
    RowBox[{"EarlyCleanupThreshold", "\[Rule]", "1"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Timing", "@", 
  RowBox[{"FixPolygons", "[", 
   RowBox[{"test", ",", 
    RowBox[{"EarlyCleanupThreshold", "\[Rule]", "2"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.41778383512042*^9, 3.41778383955768*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowSize->{667, 874},
WindowMargins->{{Automatic, 224}, {Automatic, 48}},
ShowSelection->True,
FrontEndVersion->"6.0 for Mac OS X x86 (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 103, 1, 76, "Title"],
Cell[696, 26, 165, 3, 27, "Input"],
Cell[864, 31, 56, 1, 27, "Input"],
Cell[CellGroupData[{
Cell[945, 36, 299, 4, 67, "Section"],
Cell[1247, 42, 291, 5, 56, "Text"],
Cell[1541, 49, 127, 3, 26, "Text"],
Cell[1671, 54, 788, 22, 43, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2496, 81, 404, 5, 67, "Section"],
Cell[2903, 88, 192, 5, 41, "Text"],
Cell[3098, 95, 624, 20, 43, "Input"],
Cell[CellGroupData[{
Cell[3747, 119, 120, 1, 34, "Subsection"],
Cell[3870, 122, 638, 13, 71, "Text"],
Cell[4511, 137, 571, 17, 58, "Input"],
Cell[5085, 156, 918, 25, 58, "Input"],
Cell[6006, 183, 463, 13, 58, "Input"],
Cell[6472, 198, 463, 13, 58, "Input"],
Cell[6938, 213, 349, 6, 41, "Text"],
Cell[7290, 221, 1517, 32, 118, "Input"],
Cell[8810, 255, 188, 4, 26, "Text"],
Cell[9001, 261, 1508, 42, 88, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
